
import ID;
import Rand;
import SpawnableRectangle;
import OutsideBoundingBoxException;
import IllegalRectangleArgumentsException;
import RectangleTooSmallToSpawnException;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Vector;

class AnimatedBox
	{
	/*
	Define objects that can paint themselves to a given bitmap, change
	their state, and create new ones of their kind. AnimatedBox objects
	paint themselves as rectangles of various positions, dimensions,
	and colors.

	Public Methods:
	---------------
	public void getRelatives(Vector relatives)
	public Vector reproduce(int numBoxes)
	public void changeState()
	public void paint(Graphics graphicsContext)
	*/

	//my state is defined by my id, rectangle, color, and whether i'm
	//filled or unfilled, visible or invisible
	protected ID id;
	protected SpawnableRectangle rectangle;
	protected Color color;
	protected boolean filled, visible;

	//plus a vector of all my related AnimatedBoxes
	protected Vector relatives;

	//plus an array of colors to choose from
	protected static final Color[] colors =
		{Color.red, Color.orange, Color.yellow, Color.green,
		Color.blue, Color.cyan, Color.magenta, Color.pink};

	//plus a vector of sounds to choose from
	protected static Vector sounds;

	//and the min dimensions my rectangle must have to be able to spawn another
	protected final static int MIN_SPAWN_WIDTH  = 50;
	protected final static int MIN_SPAWN_HEIGHT = 50;

	////////////////////////////////////////////////////////////////////
	public AnimatedBox(int xMin, int yMin, int width, int height)
		{
		//get my unique id
		id = ID.getID();

		//choose my color
		color = colors[Rand.integer(0, colors.length - 1)];

		//initially i'm unfilled and visible
		filled = false; visible = true;

		//generate my rectangle
		rectangle = createSpawnableRectangle(xMin, yMin, width, height);
		}

	////////////////////////////////////////////////////////////////////
	public AnimatedBox(int xMin, int yMin, int width, int height, Vector sounds)
		{
		this(xMin, yMin, width, height);

		//save the sounds vector
		this.sounds = sounds;
		}

	////////////////////////////////////////////////////////////////////
	public Vector reproduce(int numBoxes)
		{
		/*
		Produce numBoxes BouncingBoxes and return a vector containing myself
		and all created boxes.
		*/

		if (numBoxes <= 0)
			return null;

		//create the vector to return and add myself to it
		Vector boxes = new Vector(); boxes.addElement(this);

		//keep track of any AnimatedBoxes that are big enough to spawn
		Vector spawnables = new Vector(); spawnables.addElement(this);

		//generate numBoxes bouncing boxes
		int j; AnimatedBox parent; SpawnableRectangle rectangle; BouncingBox box;
		for (int i = 0; i < numBoxes; i++)
			{
			//choose a random spawnable box
			j = Rand.integer(0, spawnables.size() - 1);
			parent = (AnimatedBox) spawnables.elementAt(j);

			//ask it to spawn a rectangle inside it
			rectangle = createSpawn(parent.rectangle);

			//create the corresponding bouncing box
			box = createBouncingBox(rectangle, parent);

			//add it to the vector of bouncing boxes so far created
			boxes.addElement(box);

			//and remember if it is itself big enough to spawn
			if ((box.rectangle.getWidth() > MIN_SPAWN_WIDTH) &&
				(box.rectangle.getHeight() > MIN_SPAWN_HEIGHT))
				spawnables.addElement(box);
			}

		return boxes;
		}

	////////////////////////////////////////////////////////////////////
	public void getRelatives(Vector relatives)
		{
		/*
		Get the vector of all my relatives, if any.
		*/

		this.relatives = relatives;
		}

	////////////////////////////////////////////////////////////////////
	public void changeState()
		{
		/*
		Change my state.
		(This method is empty here; it's to be overridden by subclasses.)
		*/
		}

	////////////////////////////////////////////////////////////////////
	public void paint(Graphics graphicsContext)
		{
		/*
		Paint myself into graphicsContext (which is usually the context of
		an offscreen bitmap).
		*/

		if (visible)
			{
			graphicsContext.setColor(color);

			if (filled)
				graphicsContext.fillRect(rectangle.getXMin(), rectangle.getYMin(),
										rectangle.getWidth(), rectangle.getHeight());
			else
				graphicsContext.drawRect(rectangle.getXMin(), rectangle.getYMin(),
										rectangle.getWidth(), rectangle.getHeight());
			}
		}

	////////////////////////////////////////////////////////////////////
	protected SpawnableRectangle createSpawnableRectangle(int xMin, int yMin,
																			int width, int height)
		{
		/*
		Create a new SpawnableRectangle.
		(This is in a method to isolate the exception handling code.)
		*/

		SpawnableRectangle rectangle = null;
		try {rectangle = new SpawnableRectangle(xMin, yMin, width, height);}
		catch (IllegalRectangleArgumentsException exception)
			{System.out.println("AnimatedBox: " + exception); System.exit(1);}

		return rectangle;
		}
	////////////////////////////////////////////////////////////////////
	protected SpawnableRectangle createSpawn(SpawnableRectangle rectangle)
		{
		/*
		Create a new SpawnableRectangle.
		(This is in a method to isolate the exception handling code.)
		*/

		SpawnableRectangle newRectangle = null;
		try {newRectangle = (SpawnableRectangle) rectangle.spawn();}
		catch (RectangleTooSmallToSpawnException exception)
			{System.out.println("AnimatedBox: " + exception); System.exit(1);}

		return newRectangle;
		}

	////////////////////////////////////////////////////////////////////
	protected BouncingBox createBouncingBox(SpawnableRectangle rectangle,
														AnimatedBox parent)
		{
		/*
		Create a new BouncingBox.
		(This is in a method to isolate the exception handling code.)
		*/

		BouncingBox box = null;
		try {box = new BouncingBox(rectangle, parent);}
		catch (OutsideBoundingBoxException exception)
			{System.out.println("AnimatedBox: " + exception); System.exit(1);}

		return box;
		}
	}
