
import GraphicsPane;

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Panel;
import java.awt.Image;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Button;
import java.awt.Insets;
import java.awt.TextField;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class AnimationInterfaceShell extends Applet
	implements ActionListener
	{
	/*
	An object of this class interfaces between a user and an animator object.
	It puts a control pad full of labeled buttons and a graphics pane on the
	screen. Its graphics pane displays the latest image in the animation being
	produced by some animator object, and its control pad lets the user control
	the state of the animation being displayed.
	
	On initialization, it instantiates the animator object and gets its display
	size. Once it has that size, it adds a little extra to accommodate its
	controls, then displays its controls plus a blank image of the same size as
	the animation images to be later produced by the animator object. Then it
	waits for user commands, which it passes on to the animator object, and for
	successive images from the animator object, which it paints to the screen.

	From its point of view, the user is an image consumer and command generator,
	and the animator object is an image generator and a command consumer. It
	interfaces between the two, cutting each off from knowledge of the other.

	User <---> this interface object <---> the animator object

	This class should be subclassed to produce behaviour more specific to a
	particular class of animator objects. In particular, subclasses should
	choose a subset of the available buttons to be in consonance with the
	behaviours actually supported by a particular animator object, and they
	might also be more specific with respect to that animator object in other
	ways. However, animator objects must not know that they are not painting
	directly to the screen, and users must not know that they are not talking
	directly to an animator object.
	
	Notes:
	(1) This class could be generalized to allow more than one animator
	object producing animations simultaneously.
	(2) The animator object should be able to change its behaviour in some
	way but that need not be necessary. In the simplest case, the animator may
	only produce succesive images once started, so the only buttons possible
	would be "Start" and "Exit".
	*/

	//booleans determining which control pad buttons were requested
	protected boolean startButton   = true;
	protected boolean stopButton    = true;
	protected boolean pauseButton   = true;
	protected boolean resumeButton  = true;
	protected boolean rewindButton  = true;
	protected boolean restartButton = true;
	protected boolean runButton     = true;
	protected boolean saveButton    = true;
	protected boolean restoreButton = true;
	protected boolean exitButton    = true;

	//labels of those control pad buttons
	protected String startButtonLabel   = "Start";
	protected String stopButtonLabel    = "Stop";
	protected String pauseButtonLabel   = "Pause";
	protected String resumeButtonLabel  = "Resume";
	protected String rewindButtonLabel  = "Rewind";
	protected String restartButtonLabel = "Restart";
	protected String runButtonLabel     = "Run";
	protected String saveButtonLabel    = "Save";
	protected String restoreButtonLabel = "Restore";
	protected String exitButtonLabel    = "Exit";

	//the control pad to hold the requested buttons
	protected Panel controlPad;
	protected Color padColor = Color.darkGray;
	protected int padWidth = 200, padHeight = 100;

	//the graphics pane to display successive images of the animation
	protected GraphicsPane graphicsPane;
	protected Color paneColor = Color.black;
	protected int paneWidth = 500, paneHeight = 400;

	//the animator object that produces the animation
	protected Applet animator;
	protected int animatorWidth = 500, animatorHeight = 400;

	//my default window color and dimensions
	protected Color windowColor = Color.lightGray;
	protected int windowWidth = 500, windowHeight = 500;

	//do i need double-buffering??
	protected Image offscreenImage;
	protected Graphics offscreenGraphics;

	//////////////////////////////////////////////////////////////////////
	public void init()
		{
		//create the animator object and get its dimensions
		createAnimator();

		//get and set my window dimensions
		getWindowDimensions();

		//setup my control pad
		setupControlPad();

		//setup my graphics pane
		setupGraphicsPane();

		//setup my window
		setupWindow();

		//initialize my graphics pane
		graphicsPane.init();

		//okay, everything's set; tell the animator object to begin animating
		runAnimator();
		}

	//////////////////////////////////////////////////////////////////////
	public void update(Graphics graphicsContext)
		{
		System.out.println("AnimationInterfaceShell.update()");

		//to reduce flicker
		paint(graphicsContext);
		}

	//////////////////////////////////////////////////////////////////////
	public void repaint(Graphics graphicsContext)
		{
		System.out.println("AnimationInterfaceShell.repaint()");

		graphicsPane.repaint();
		super.repaint();
		}

	//////////////////////////////////////////////////////////////////////
	public void paint(Graphics graphicsContext)
		{
		System.out.println("AnimationInterfaceShell.paint()");

		//graphicsContext.drawImage(offscreenImage, 0, 0, this);
		}

	//////////////////////////////////////////////////////////////////////
	protected void offscreenPaint(Graphics graphicsContext)
		{
		System.out.println("AnimationInterfaceShell.offscreenPaint()");
		}

	//////////////////////////////////////////////////////////////////////
	protected void createAnimator()
		{
		/*
		Create the animator object and get its dimensions.
		(This method is empty here, it is to be overridden in subclasses.)
		*/
		}

	//////////////////////////////////////////////////////////////////////
	protected void runAnimator()
		{
		/*
		Run the animator object.
		(This method is empty here, it is to be overridden in subclasses.)
		*/
		}

	//////////////////////////////////////////////////////////////////////
	protected void getWindowDimensions()
		{
		/*
		Get my window dimensions, if they're set in the html page; if not,
		use the defaults.
		*/

		String tempString;

		tempString = this.getParameter("width");
		if (tempString != null)
			{
			try
				{
				windowWidth = Integer.parseInt(tempString);
				}
			catch (NumberFormatException ignored)
				{}
			}

		tempString = this.getParameter("height");
		if (tempString != null)
			{
			try
				{
				windowHeight = Integer.parseInt(tempString);
				}
			catch (NumberFormatException ignored)
				{}
			}
		}

	//////////////////////////////////////////////////////////////////////
	protected void setupControlPad()
		{
		/*
		Setup my control pad's color and size.
		*/

		//create the control pad to hold all the buttons
		controlPad = new Panel();
		controlPad.setLayout(new FlowLayout());

		//add the requested buttons to it
		addButtons();

		//make it as big as it wants to be
		Dimension panelDimension = controlPad.getPreferredSize();
		padWidth = panelDimension.width; padHeight = panelDimension.height;
		controlPad.setSize(padWidth, padHeight);

		controlPad.setBackground(padColor);

		//add it to my window
		this.add(controlPad);
		}

	//////////////////////////////////////////////////////////////////////
	protected void setupGraphicsPane()
		{
		/*
		Setup my graphics pane's color and size.
		*/

		//create the graphics pane to display the animation
		graphicsPane = new GraphicsPane(this);

		//make it as big as the animation wants to be
		paneWidth = animatorWidth; paneHeight = animatorHeight;
		graphicsPane.setSize(paneWidth, paneHeight);

		graphicsPane.setBackground(paneColor);

		//add it to my window
		this.add(graphicsPane);
		}

	//////////////////////////////////////////////////////////////////////
	protected void setupWindow()
		{
		/*
		Setup my window's color and size.
		*/

		//make my window big enough to hold the pad and the pane
		windowWidth = Math.max(padWidth, paneWidth);
		windowHeight = Math.max(windowHeight, padHeight + paneHeight);

		//and make sure none of it is obscured by java's pathetic windowing
		//(unfortunately, java still ignores this...)
		TextField statusBar = new TextField(); //to simulate status bar sizing
		Insets insets = this.getInsets();
		this.setSize((windowWidth + insets.left + insets.right),
						(windowHeight + insets.top + insets.bottom +
						statusBar.getPreferredSize().height));

		this.setBackground(windowColor);
		}

	//////////////////////////////////////////////////////////////////////
	protected void addButtons()
		{
		/*
		Add the requested buttons to the control pad
		*/

		if (startButton)
			{
			Button startButton = new Button(startButtonLabel);
			startButton.addActionListener(this);
			controlPad.add(startButton);
			}
		if (stopButton)
			{
			Button stopButton = new Button(stopButtonLabel);
			stopButton.addActionListener(this);
			controlPad.add(stopButton);
			}
		if (pauseButton)
			{
			Button pauseButton = new Button(pauseButtonLabel);
			pauseButton.addActionListener(this);
			controlPad.add(pauseButton);
			}
		if (resumeButton)
			{
			Button resumeButton = new Button(resumeButtonLabel);
			resumeButton.addActionListener(this);
			controlPad.add(resumeButton);
			}
		if (rewindButton)
			{
			Button rewindButton = new Button(rewindButtonLabel);
			rewindButton.addActionListener(this);
			controlPad.add(rewindButton);
			}
		if (restartButton)
			{
			Button restartButton = new Button(restartButtonLabel);
			restartButton.addActionListener(this);
			controlPad.add(restartButton);
			}
		if (runButton)
			{
			Button runButton = new Button(runButtonLabel);
			runButton.addActionListener(this);
			controlPad.add(runButton);
			}
		if (saveButton)
			{
			Button saveButton = new Button(saveButtonLabel);
			saveButton.addActionListener(this);
			controlPad.add(saveButton);
			}
		if (restoreButton)
			{
			Button restoreButton = new Button(restoreButtonLabel);
			restoreButton.addActionListener(this);
			controlPad.add(restoreButton);
			}
		if (exitButton)
			{
			Button exitButton = new Button(exitButtonLabel);
			exitButton.addActionListener(this);
			controlPad.add(exitButton);
			}
		}

	//////////////////////////////////////////////////////////////////////
	public void actionPerformed(ActionEvent event)
		{
		/*
		Distribute the button presses to the appropriate handler code.
		*/

		String buttonLabel = event.getActionCommand();

		if (buttonLabel.equals(startButtonLabel))
			{
			System.out.println("start button pressed");

			handleStartButton();
			this.repaint();
			if (graphicsPane != null)
				graphicsPane.repaint();
			return;
			}

		if (buttonLabel.equals(stopButtonLabel))
			{
			System.out.println("stop button pressed");

			handleStopButton();
			this.repaint();
			if (graphicsPane != null)
				graphicsPane.repaint();
			return;
			}

		if (buttonLabel.equals(pauseButtonLabel))
			{
			System.out.println("pause button pressed");

			handlePauseButton();
			this.repaint();
			if (graphicsPane != null)
				graphicsPane.repaint();
			return;
			}

		if (buttonLabel.equals(resumeButtonLabel))
			{
			System.out.println("resume button pressed");

			handleResumeButton();
			this.repaint();
			if (graphicsPane != null)
				graphicsPane.repaint();
			return;
			}

		if (buttonLabel.equals(rewindButtonLabel))
			{
			System.out.println("rewind button pressed");

			handleRewindButton();
			this.repaint();
			if (graphicsPane != null)
				graphicsPane.repaint();
			return;
			}

		if (buttonLabel.equals(restartButtonLabel))
			{
			System.out.println("restart button pressed");

			handleRestartButton();
			this.repaint();
			if (graphicsPane != null)
				graphicsPane.repaint();
			return;
			}

		if (buttonLabel.equals(runButtonLabel))
			{
			System.out.println("run button pressed");

			handleRunButton();
			this.repaint();
			if (graphicsPane != null)
				graphicsPane.repaint();
			return;
			}

		if (buttonLabel.equals(saveButtonLabel))
			{
			System.out.println("save button pressed");

			handleSaveButton();
			this.repaint();
			if (graphicsPane != null)
				graphicsPane.repaint();
			return;
			}

		if (buttonLabel.equals(restoreButtonLabel))
			{
			System.out.println("restore button pressed");

			handleRestoreButton();
			this.repaint();
			if (graphicsPane != null)
				graphicsPane.repaint();
			return;
			}

		if (buttonLabel.equals(exitButtonLabel))
			{
			System.out.println("exit button pressed");

			handleExitButton();
			this.repaint();
			if (graphicsPane != null)
				graphicsPane.repaint();
			return;
			}
		}

	//////////////////////////////////////////////////////////////////////
	protected void handleStartButton()
		{
		System.out.println("AnimationInterfaceShell.handleStartButton()");
		}

	//////////////////////////////////////////////////////////////////////
	protected void handleStopButton()
		{
		System.out.println("AnimationInterfaceShell.handleStopButton()");
		}

	//////////////////////////////////////////////////////////////////////
	protected void handlePauseButton()
		{
		System.out.println("AnimationInterfaceShell.handlePauseButton()");
		}

	//////////////////////////////////////////////////////////////////////
	protected void handleResumeButton()
		{
		System.out.println("AnimationInterfaceShell.handleResumeButton()");
		}

	//////////////////////////////////////////////////////////////////////
	protected void handleRewindButton()
		{
		System.out.println("AnimationInterfaceShell.handleRewindButton()");
		}

	//////////////////////////////////////////////////////////////////////
	protected void handleRestartButton()
		{
		System.out.println("AnimationInterfaceShell.handleRestartButton()");
		}

	//////////////////////////////////////////////////////////////////////
	protected void handleRunButton()
		{
		System.out.println("AnimationInterfaceShell.handleRunButton()");
		}

	//////////////////////////////////////////////////////////////////////
	protected void handleSaveButton()
		{
		System.out.println("AnimationInterfaceShell.handleSaveButton()");
		}

	//////////////////////////////////////////////////////////////////////
	protected void handleRestoreButton()
		{
		System.out.println("AnimationInterfaceShell.handleRestoreButton()");
		}

	//////////////////////////////////////////////////////////////////////
	protected void handleExitButton()
		{
		System.out.println("AnimationInterfaceShell.handleExitButton()");
		}
	}
