
import ClockObserver;
import Clock;

import java.applet.Applet;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Color;

public class AnimationShell extends Applet
	implements ClockObserver
	{
	/*
	This applet shell can be subclassed to produce animation applets.
	It uses an offscreen image to double buffer the animation and so
	avoid image flicker and it observes a clock that executes its tick()
	method to control when the animation is redrawn. It also lets subclasses
	control how the animation is drawn (whether it's to be paused, resumed,
	restarted, or killed).

	Public Methods:
	---------------
	init()
	start()
	stop()
	run()
	update(graphicsContext)
	paint(graphicsContext)
	tick()
	pauseAnimation()
	resumeAnimation()
	restartAnimation()
	endAnimation()
	*/

	//my offscreen bit map and its graphics context
	protected Image offscreenImage;
	protected Graphics offscreenGraphics;

	//my clock and its speed
	Clock clock;
	protected int CLOCK_TICKS_PER_SECOND = 25;

	//my window's background color, dimensions, and its minimum dimensions
	protected Color backgroundColor = Color.black;
	protected int windowWidth, windowHeight;
	protected static int MIN_WINDOW_WIDTH  = 50;
	protected static int MIN_WINDOW_HEIGHT = 50;

	//the animation controller
	protected boolean paused;

	////////////////////////////////////////////////////////////////////
	public void init()
		{
		/*
		Setup my onscreen and offscreen bitmaps.
		*/

		//fetch my window's dimensions and blank the screen
		//(make my window at least MIN_WINDOW_WIDTHxMIN_WINDOW_WIDTH pixels)
		windowWidth = this.getSize().width;
		windowHeight = this.getSize().height;
		windowWidth = Math.max(windowWidth, MIN_WINDOW_WIDTH);
		windowHeight = Math.max(windowHeight, MIN_WINDOW_HEIGHT);
		this.setBackground(backgroundColor);

		//setup my offscreen bitmap
		offscreenImage = this.createImage(windowWidth, windowHeight);
		offscreenGraphics = offscreenImage.getGraphics();
		offscreenGraphics.setColor(backgroundColor);
		offscreenGraphics.fillRect(0, 0, windowWidth, windowHeight);

		//start my clock
		clock = new Clock(CLOCK_TICKS_PER_SECOND, this);

		//start the animation
		this.start();
		}

	////////////////////////////////////////////////////////////////////
	public void start()
		{
		/*
		Start the animation.
		*/

		paused = false;
		}

	////////////////////////////////////////////////////////////////////
	public void stop()
		{
		/*
		Stop the animation.
		*/

		paused = true;
		}

	////////////////////////////////////////////////////////////////////
	public void tick()
		{
		/*
		This method is regularly executed by my clock.
		*/

		if (! paused)
			run();
		}

	////////////////////////////////////////////////////////////////////
	public void run()
		{
		/*
		Generate the next animation frame and paint it.
		*/

		updateAnimation();
		this.repaint();
		}

	////////////////////////////////////////////////////////////////////
	public void pauseAnimation()
		{
		this.stop();
		}

	////////////////////////////////////////////////////////////////////
	public void resumeAnimation()
		{
		this.start();
		}

	////////////////////////////////////////////////////////////////////
	public void restartAnimation()
		{
		/*
		Start the animation over from the beginning.
		*/

		this.stop();
		this.init();
		}

	////////////////////////////////////////////////////////////////////
	public void endAnimation()
		{
		/*
		Kill the animation dead.
		*/

		this.stop();
		offscreenImage = null;
		offscreenGraphics = null;
		clock.die(this);
		clock = null;
		}

	////////////////////////////////////////////////////////////////////
	public void update(Graphics graphicsContext)
		{
		/*
		Overridde the default Applet.update() to decrease flicker.
		*/

		paint(graphicsContext);
		}

	////////////////////////////////////////////////////////////////////
	public void paint(Graphics graphicsContext)
		{
		/*
		Paint my offscreen bitmap to my window.
		*/

		//blank my offscreen bitmap
		offscreenGraphics.setColor(backgroundColor);
		offscreenGraphics.fillRect(0, 0, windowWidth, windowHeight);

		//paint the current animation frame into it
		paintAnimationFrame(offscreenGraphics);

		//draw it to the screen
		graphicsContext.drawImage(offscreenImage, 0, 0, this);
		}

	////////////////////////////////////////////////////////////////////
	protected void updateAnimation()
		{
		/*
		Update the animation that i will later paint to my offscreen bitmap.

		This method does nothing here; it's to be overridden by subclasses.
		*/
		}

	////////////////////////////////////////////////////////////////////
	protected void paintAnimationFrame(Graphics graphicsContext)
		{
		/*
		Paint a frame of the animation into graphicsContext.

		This method does nothing here; it's to be overridden by subclasses.
		*/
		}
	}
