
import Tuple;
import Rand;
import AnimatedBox;
import SpawnableRectangle;
import OutsideBoundingBoxException;

import java.awt.Point;
import java.util.Vector;
import java.util.Enumeration;
import java.applet.AudioClip;

class BouncingBox extends AnimatedBox
	{
	/*
	Bounce around inside a box, which may also be bouncing.

	Public Methods:
	---------------
	public void getRelatives(Vector boxes)
	public void changeState()

	Protected Methods:
	------------------
	protected void handleInsideCollision(AnimatedBox otherBox)
	protected boolean newCollision(AnimatedBox otherBox)
	protected void recoilX(AnimatedBox otherBox, double displacement)
	protected void recoilY(AnimatedBox otherBox, double displacement)
	*/

	//my state is the same as all AnimatedBoxes...

	//plus my parent AnimatedBox (which defines my bounding box),
	//any sibling bouncing boxes also bouncing around inside my parent,
	//and my bounce sound
	protected AnimatedBox parent;
	protected Vector siblings;
	protected AudioClip sound;

	//plus whether i'm in contact with another box
	boolean contact;

	//////////////////////////////////////////////////////////////////
	public BouncingBox(SpawnableRectangle rectangle, AnimatedBox parent)
							throws OutsideBoundingBoxException
		{
		/*
		Create a bouncing box inside an AnimatedBox
		(which may also be a BouncingBox).

		Note: reusing a single velocity object for all boxes
		links their velocities and makes for an interesting display.
		*/

		super(rectangle.getXMin(), rectangle.getYMin(),
			rectangle.getWidth(), rectangle.getHeight());

		//make sure i'm inside my bounding box
		if (! parent.rectangle.surrounds(this.rectangle))
			throw new OutsideBoundingBoxException(id);

		//remember my parent AnimatedBox
		this.parent = parent;

		//if there are bounce sounds, randomly choose one as my bounce sound
		if ((sounds != null) && (sounds.size() > 0))
			{
			sound = (AudioClip)
				sounds.elementAt(Rand.integer(0, sounds.size() - 1));
			}

		//initially i'm not touching my parent
		contact = false;
		}

	//////////////////////////////////////////////////////////////////
	public void getRelatives(Vector relatives)
		{
		/*
		Fetch the vector of all AnimatedBoxes and check it for my siblings.

		If a box isn't me, and it's a Bouncing Box, and we have the same
		parent, then it's a sibling.
		*/

		siblings = new Vector(); AnimatedBox box;
		Enumeration list = relatives.elements();
		while (list.hasMoreElements())
			{
			box = (AnimatedBox) list.nextElement();
			if (box.id.notEquals(this.id) &&
				(box instanceof BouncingBox) &&
				(((BouncingBox) box).parent == this.parent))
					siblings.addElement(box);
			}
		}

	//////////////////////////////////////////////////////////////////
	public void changeState()
		{
		/*
		Move myself depending on my velocity and whether i've hit my parent
		or any of my siblings.

		If siblings == null, then i'm allowed to interpenetrate any siblings
		i may have.
		*/

		filled = false;

		//NOTE: SHOULD PLAY A SOUND IMMEDIATELY IF I DETECT THAT I"M IN CONTACT
		//WITH ANYONE. THEN MOVE!

		rectangle.moveSelf();

		//check for collision with my parent; this must be handled separately
		//since i must always be inside my parent
		handleInsideCollision(parent);

		//if i'm not allowed to interpenetrate my sibling bouncing boxes
		//check for collision with any of them
		if (siblings != null)
			{
			BouncingBox sib;
			Enumeration list = siblings.elements();
			while (list.hasMoreElements())
				{
				sib = (BouncingBox) list.nextElement();
				if (this.rectangle.intersects(sib.rectangle))
					//rectangle.handleHit(sib.rectangle);
				}
			}
		}

	//////////////////////////////////////////////////////////////////
	protected boolean newCollision(AnimatedBox otherBox)
		{
		/*
		Determine whether this is a new collision or not.
		(i.e. whether i'm currently in continuous contact with otherBox)
		*/

		if ((
			((rectangle.getXMax() >= otherBox.rectangle.getXMax()) &&
			(rectangle.velocity.x > 0)) ||
			((rectangle.getYMax() >= otherBox.rectangle.getYMax()) &&
			(rectangle.velocity.y > 0)) ||
			((rectangle.getXMin() <= otherBox.rectangle.getXMin()) &&
			(rectangle.velocity.x < 0)) ||
			((rectangle.getYMin() <= otherBox.rectangle.getYMin()) &&
			(rectangle.velocity.y < 0))) &&
			! contact)
			return true;
		else
			return false;
		}

	//////////////////////////////////////////////////////////////////
	protected void handleInsideCollision(AnimatedBox otherBox)
		{
		/*
		I've hit one of otherBox's walls; play a sound and move away.
		Stay WITHIN otherBox.
		*/

		//NOTE: no sound is played if my parent hit me!
		if (newCollision(otherBox))
			{
			if (sound != null) sound.play();
			//filled = true; //to blink on impact...
			contact = true;
			}

		double displacement;

		//check for collisions with my container's right vertical,
		//left vertical, lower horizontal, and upper horizontal walls.

		displacement = this.rectangle.getXMax() - otherBox.rectangle.getXMax();
		if (displacement >= 0)
			recoilX(otherBox, displacement);

		displacement = this.rectangle.getXMin() - otherBox.rectangle.getXMin();
		if (displacement <= 0)
			recoilX(otherBox, displacement);

		displacement = this.rectangle.getYMax() - otherBox.rectangle.getYMax();
		if (displacement >= 0)
			recoilY(otherBox, displacement);

		displacement = this.rectangle.getYMin() - otherBox.rectangle.getYMin();
		if (displacement <= 0)
			recoilY(otherBox, displacement);

		//the two boxes are no longer in contact
		contact = false;
		}

	//////////////////////////////////////////////////////////////////
	protected void recoilX(AnimatedBox otherBox, double displacement)
		{
		/*
		I've hit a wall of otherBox and i'm displacement pixels past one
		of its vertical walls, bounce off.
		*/

		double totalMass, massDifference;
		double myVelocity, otherVelocity;
		BouncingBox tempBox;

		//recoil in different ways depending on whether otherBox is also
		//a BouncingBox or not
		if (otherBox instanceof BouncingBox)
			{
			//setup convenience variables
			tempBox = (BouncingBox) otherBox;
			totalMass = tempBox.rectangle.mass + this.rectangle.mass;
			massDifference = tempBox.rectangle.mass - this.rectangle.mass;
			myVelocity = this.rectangle.velocity.x;
			otherVelocity = tempBox.rectangle.velocity.x;

			//displace both boxes
			this.rectangle.moveX(-displacement * tempBox.rectangle.mass
				/ totalMass);
			tempBox.rectangle.moveX(displacement * this.rectangle.mass
				/ totalMass);

			//modify velocities after elastic collision
			this.rectangle.velocity.x =
				(2 * tempBox.rectangle.mass * otherVelocity -
					massDifference * myVelocity) / totalMass;
			tempBox.rectangle.velocity.x =
				(2 * this.rectangle.mass * myVelocity +
					massDifference * otherVelocity) / totalMass;
			}
		else
			{
			//otherBox is stationary; it's mass is assumed to be infinite
			//and its velocity is zero, so only alter my position and velocity
			this.rectangle.moveX(-displacement);
			this.rectangle.velocity.x = -this.rectangle.velocity.x;
			}
		}

	//////////////////////////////////////////////////////////////////
	protected void recoilY(AnimatedBox otherBox, double displacement)
		{
		/*
		I've hit a wall of otherBox and i'm displacement pixels past one
		of its horizontal walls, bounce off.
		*/

		double totalMass, massDifference;
		double myVelocity, otherVelocity;
		BouncingBox tempBox;

		//recoil in different ways depending on whether otherBox is also
		//a BouncingBox or not
		if (otherBox instanceof BouncingBox)
			{
			//setup convenience variables
			tempBox = (BouncingBox) otherBox;
			totalMass = tempBox.rectangle.mass + this.rectangle.mass;
			massDifference = tempBox.rectangle.mass - this.rectangle.mass;
			myVelocity = this.rectangle.velocity.y;
			otherVelocity = tempBox.rectangle.velocity.y;

			//displace both boxes
			this.rectangle.moveY(-displacement * tempBox.rectangle.mass
				/ totalMass);
			tempBox.rectangle.moveY(displacement * this.rectangle.mass
				/ totalMass);

			//modify velocities after elastic collision
			this.rectangle.velocity.y =
				(2 * tempBox.rectangle.mass * otherVelocity -
					massDifference * myVelocity) / totalMass;
			tempBox.rectangle.velocity.y =
				(2 * this.rectangle.mass * myVelocity +
					massDifference * otherVelocity) / totalMass;
			}
		else
			{
			//otherBox is stationary; it's mass is assumed to be infinite
			//and its velocity is zero, so only alter my position and velocity
			this.rectangle.moveY(-displacement);
			this.rectangle.velocity.y = -this.rectangle.velocity.y;
			}
		}
	}
