
import java.lang.Runnable;
import java.lang.Thread;
import java.util.Vector;
import java.util.Enumeration;

class Clock
	implements Runnable
	{
	/*
	Define an object that regularly executes the tick() method of all
	its observers.

	Objects become observers by executing this clock's addClockObserver()
	method or by passing themselves to this clock on its creation.

	Public Constructors & Methods:
	------------------------------
	Clock()
	Clock(ticksPerSecond)
	Clock(ticksPerSecond, observer)
	addClockObserver(observer)
	deleteClockObserver(observer)
	start()
	stop()
	run()
	timeSinceLastTick()
	*/

	//my thread, observer list, clock rate, and temporary variables
	protected Thread ticker;
	protected Vector observers;
	protected int ticksPerSecond = 25;
	protected long sleepTime = 40;
	protected long startTickTime, currentTickTime, lastTickTime, tickCount;

	///////////////////////////////////////////////////////////////////////
	public Clock()
		{
		startTickTime = 0; currentTickTime = 0; lastTickTime = 0; tickCount = 0;

		//initialize my thread and my observers list, and start myself up
		ticker = new Thread(this);
		observers = new Vector();
		this.start();
		}

	///////////////////////////////////////////////////////////////////////
	public Clock(int ticksPerSecond)
		{
		/*
		Create a clock with a given speed.
		*/

		this();
		this.ticksPerSecond = ticksPerSecond;
		sleepTime = (long) (1000 / ticksPerSecond); //converts to milliseconds
		}

	///////////////////////////////////////////////////////////////////////
	public Clock(int ticksPerSecond, ClockObserver observer)
		{
		/*
		Create a clock with a given speed and a given observer.
		*/

		this(ticksPerSecond);
		addClockObserver(observer);
		}

	///////////////////////////////////////////////////////////////////////
	public synchronized void addClockObserver(ClockObserver observer)
		{
		observers.addElement(observer);
		}

	///////////////////////////////////////////////////////////////////////
	public synchronized void deleteClockObserver(ClockObserver observer)
		{
		observers.removeElement(observer);
		}

	///////////////////////////////////////////////////////////////////////
	public synchronized void die(ClockObserver observer)
		{
		/*
		Die if requested to by my last observer.

		Note: this would leave the observer (and any careless past observers)
		with a null pointer. how to fix that without requiring them to remember
		to null out their references to me? need some way to make myself private
		while still allowing arbitrary objects to observe me. I need a
		ClockManager.
		*/

		if ((observers.size() == 1) && observers.elementAt(0).equals(observer))
			{
			this.stop();
			observers = null;
			try {this.finalize();}
			catch (Throwable ignored) {} //just die
			}
		}

	///////////////////////////////////////////////////////////////////////
	public void start()
		{
		if (ticker == null)
			ticker = new Thread(this);
		ticker.start();

		startTickTime = System.currentTimeMillis();
		lastTickTime = startTickTime;
		currentTickTime = startTickTime;
		}

	///////////////////////////////////////////////////////////////////////
	public void stop()
		{
		if (ticker != null)
			ticker.stop();
		ticker = null;
		}

	///////////////////////////////////////////////////////////////////////
	public void run()
		{
		while (ticker != null)
			{
			currentTickTime = System.currentTimeMillis();

			//execute all my observers' tick() methods
			Enumeration watchers = observers.elements();
			while (watchers.hasMoreElements())
				((ClockObserver) watchers.nextElement()).tick();
			tickCount++;

			long elapsedTime = System.currentTimeMillis() - currentTickTime;

			lastTickTime = currentTickTime;

			//the time to sleep depends on how much time it took to call all
			//the tick() methods, so sleep only long enough to keep the beat
			if (elapsedTime < sleepTime)
				{
				try {Thread.sleep(sleepTime - elapsedTime);}
				catch (InterruptedException exception)
					{
					System.out.println("Clock: something bad happened.");
					exception.printStackTrace();
					throw new RuntimeException(exception.toString());
					}
				}
			}
		}

	///////////////////////////////////////////////////////////////////////
	public long timeSinceLastTick()
		{
		return currentTickTime - lastTickTime;
		}

	///////////////////////////////////////////////////////////////////////
	protected long getTickCount()
		{
		return tickCount;
		}
	}
