
import MoveableRectangle;
import Hittable;
import IllegalRectangleArgumentsException;
import Tuple;
import Rand;

class HittableRectangle extends MoveableRectangle
	implements Hittable
	{
	/*
	A HittableRectangle has mass and velocity. It may hit things and
	may be hit by things. It is perfectly rigid and elastic. It obeys
	the laws of classical mechanics.

	Conservation Laws:
	------------------
	-conservation of momentum
	-conservation of kinetic energy
	-conservation of angular momentum (NOT YET IMPLEMENTED)

	Newton's Laws:
	--------------
	-continue whatever you doing until acted upon by an outside force
	-react to a force with an acceleration proportional to your mass
	*/

	//my mass, centroid, velocity, and momentum
	protected int mass;
	protected Tuple centroid;
	protected Tuple velocity;
	protected Tuple momentum;

	//initial range of x, y velocities of any self-generated HittableRectangle
	protected final static int MIN_X_VELOCITY = -4;
	protected final static int MAX_X_VELOCITY = +4;
	protected final static int MIN_Y_VELOCITY = -4;
	protected final static int MAX_Y_VELOCITY = +4;

	///////////////////////////////////////////////////////////////////
	public HittableRectangle(int xMin, int yMin, int width, int height)
		throws IllegalRectangleArgumentsException
		{
		/*
		Generate a randomly moving HittableRectangle.
		*/

		super(xMin, yMin, width, height);

		//randomly generate my initial velocity
		this.velocity =
			new Tuple(Rand.integer(MIN_X_VELOCITY, MAX_X_VELOCITY),
						Rand.integer(MIN_Y_VELOCITY, MAX_Y_VELOCITY));

		//calculate my mass, momentum, and centroid
		mass = this.getPerimeter();
		momentum = new Tuple(mass * velocity.x, mass * velocity.y);
		centroid = new Tuple((xMin + xMax) / 2, (yMin + yMax) / 2);
		}

	///////////////////////////////////////////////////////////////////
	public HittableRectangle(int xMin, int yMin, int width, int height,
									Tuple velocity)
		throws IllegalRectangleArgumentsException
		{
		super(xMin, yMin, width, height);

		//remember my initial velocity
		this.velocity = velocity;

		//calculate my mass, momentum, and centroid
		mass = this.getPerimeter();
		momentum = new Tuple(mass * velocity.x, mass * velocity.y);
		centroid = new Tuple((xMin + xMax) / 2, (yMin + yMax) / 2);
		}

	///////////////////////////////////////////////////////////////////
	public int getMass()
		{
		/*
		Return my mass.
		*/

		return mass;
		}

	///////////////////////////////////////////////////////////////////
	public Tuple getCentroid()
		{
		/*
		Return my centroid.
		*/

		return centroid;
		}

	///////////////////////////////////////////////////////////////////
	public double getXVelocity()
		{
		/*
		Return my horizontal velocity.
		*/

		return velocity.x;
		}

	///////////////////////////////////////////////////////////////////
	public double getYVelocity()
		{
		/*
		Return my vertical velocity.
		*/

		return velocity.y;
		}

	///////////////////////////////////////////////////////////////////
	public Tuple getVelocity()
		{
		/*
		Return my velocity.
		*/

		return velocity;
		}

	///////////////////////////////////////////////////////////////////
	public void setVelocity(Tuple velocity)
		{
		/*
		Reset my velocity (and of course my momentum).
		*/

		this.velocity = velocity;
		this.momentum.x = mass * velocity.x;
		this.momentum.y = mass * velocity.y;
		}

	///////////////////////////////////////////////////////////////////
	public double getXMomentum()
		{
		/*
		Return my horizontal momentum.
		*/

		return momentum.x;
		}

	///////////////////////////////////////////////////////////////////
	public double getYMomentum()
		{
		/*
		Return my vertical momentum.
		*/

		return momentum.y;
		}

	///////////////////////////////////////////////////////////////////
	public Tuple getMomentum()
		{
		/*
		Return my momentum.
		*/

		return momentum;
		}

	///////////////////////////////////////////////////////////////////
	public void moveSelf()
		{
		/*
		Move myself according to my current velocity.
		*/

		this.move(velocity.x, velocity.y);
		}

	///////////////////////////////////////////////////////////////////
	public void handleHit(Hittable object)
		{
		/*
		I've been hit by another Hittable.

		If two Hittables with masses m and n collide, their collision will
		be elastic---both kinetic energy and momentum will be conserved.
		So, if
		old velocity of mass m Hittable = a
		old velocity of mass n Hittable = b
		then after solving the two conservation equations
		new velocity of mass m Hittable = (2nb + a(m-n))/(m+n)
		new velocity of mass n Hittable = (2ma - b(m-n))/(m+n)

		Both objects experience a momentum change (and thus an impulse) of:
		2mn(a-b)/(m+n)
		*/

		//setup convenience variables
		double myMass = this.getMass();
		Tuple myVelocity = this.getVelocity();
		double itsMass = object.getMass();
		Tuple itsVelocity = object.getVelocity();

		double totalMass = myMass + itsMass;
		double massDifference = myMass - itsMass;

		double itsXMomentum = itsMass * itsVelocity.x;
		double itsYMomentum = itsMass * itsVelocity.y;

		//update my velocity
		this.velocity.x =
			(2 * itsXMomentum + myVelocity.x * massDifference) / totalMass;
		this.velocity.y =
			(2 * itsYMomentum + myVelocity.y * massDifference) / totalMass;

		//update its velocity
		Tuple itsVel = new Tuple();
		itsVel.x =
			(2 * this.momentum.x - itsVelocity.x * massDifference) / totalMass;
		itsVel.y =
			(2 * this.momentum.y - itsVelocity.y * massDifference) / totalMass;
		object.setVelocity(itsVel);

/*
System.out.println("my velocity before impact = " + myVelocity);
System.out.println("my velocity after impact = " + velocity);
System.out.println("its velocity before impact = " + itsVelocity);
System.out.println("my velocity after impact = " + itsVel);
System.out.println();
*/

		//move it and move myself
		this.moveSelf();
		object.moveSelf();
		}
	}
