
import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.StreamTokenizer;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

class HtmlParser
	{
	/*
	Parse an html file and extract any applet parameters found there.

	This class may be used as a standalone application or it may be used
	by another object to parse the first applet tag found in a given html
	file as in:
		HtmlParser parser = HtmlParser.parse(htmlFile);
	the calling object can then use parser's public accessor methods to
	fetch appletName, appletCodebase, appletWidth, and appletHeight,
	as well as any applet parameters found.

	Public Methods:
	---------------
	main()
	parse(String)
	parse(StreamTokenizer)
	getAppletName()
	getAppletWidth()
	getAppletHeight()
	getAppletParameters()
	*/

	//globals to store the applet's main variables
	protected String appletName;
	protected String appletCodebase;
	protected int appletWidth;
	protected int appletHeight;
	protected Hashtable appletParameters;

	//mnemonic forms of various StreamTokenizer constants
	protected final static int WORD        = StreamTokenizer.TT_WORD;
	protected final static int NUMBER      = StreamTokenizer.TT_NUMBER;
	protected final static int END_OF_FILE = StreamTokenizer.TT_EOF;

	///////////////////////////////////////////////////////////////////
	public static void main(String[] commandLineArguments)
		throws IOException
		{
		//create a new HtmlParser object to parse the given html file
		HtmlParser parser = HtmlParser.parse(commandLineArguments[0]);

		if (parser == null)
			{
			System.out.println("can't find or can't parse the file");
			System.exit(1);
			}

		//print any parameters found
		System.out.println("here are all the applet parameters found:");
		System.out.println("name     = " + parser.getAppletName());
		System.out.println("codebase = " + parser.getAppletCodebase());
		System.out.println("width    = " + parser.getAppletWidth());
		System.out.println("height   = " + parser.getAppletHeight());
		Enumeration names = parser.getAppletParameters().keys();
		Enumeration values = parser.getAppletParameters().elements();
		while (names.hasMoreElements())
			System.out.println(names.nextElement() + " = " + values.nextElement());
		}

	///////////////////////////////////////////////////////////////////
	public static HtmlParser parse(String htmlFile)
		{
		/*
		This method gives other classes access to this class's functionality
		without their having to explicitly instantiate an HtmlParser object.
		*/

		//create a string array pass to the filename to the parser
		String[] fileArray = new String[1];
		fileArray[0] = htmlFile;

		//connect to the requested text stream
		StreamTokenizer stream = null;
		try
			{ stream = HtmlParser.getStream(fileArray); }
		catch (IOException ignored)
			{
			//there's a problem reading the file or converting it to a stream
			return null;
			}

		//parse the stream and return the parsed arguments
		return HtmlParser.parse(stream);
		}

	///////////////////////////////////////////////////////////////////
	public static HtmlParser parse(StreamTokenizer stream)
		{
		/*
		This method gives other classes access to this class's functionality
		without their having to explicitly instantiate an HtmlParser object.
		*/

		//NOTE: how can i call this method multiple times on the same stream
		//so that i can parse mutliple applet tags??

		//create a new HtmlParser object to parse the given html stream
		HtmlParser parser = new HtmlParser();

		try
			{
			//find the first applet tag in the stream
			//(if nothing found, there's no applet tag in the stream)
			if (!parser.getNextApplet(stream))
				return null;

			//parse it into applet tag tokens
			Vector strings = parser.parseAppletTokens(stream);

			//if nothing found, the applet tag is malformed
			if (strings.size() == 0)
				return null;

			//store them in appletParameters + the other applet variables
			parser.appletParameters = parser.storeAppletParameters(strings);
			}
		catch (IOException ignored)
			{
			//there's a problem parsing the file
			return null;
			}

		//return all the applet variables found
		return parser;
		}

	///////////////////////////////////////////////////////////////////
	public String getAppletName()
		{
		return appletName;
		}

	///////////////////////////////////////////////////////////////////
	public String getAppletCodebase()
		{
		return appletCodebase;
		}

	///////////////////////////////////////////////////////////////////
	public int getAppletWidth()
		{
		return appletWidth;
		}

	///////////////////////////////////////////////////////////////////
	public int getAppletHeight()
		{
		return appletHeight;
		}

	///////////////////////////////////////////////////////////////////
	public Hashtable getAppletParameters()
		{
		return appletParameters;
		}

	///////////////////////////////////////////////////////////////////
	protected static StreamTokenizer getStream(String[] commandLineArguments)
		throws IOException
		{
		/*
		Accept text input from the command line or from an html file
		depending on whether anything was given on the command line.
		If there's any error, throw IOException.
		*/

		InputStream inData = System.in;
		if (commandLineArguments.length == 1)
			inData = new FileInputStream(new File(commandLineArguments[0]));

		return
			new StreamTokenizer(new BufferedReader(new InputStreamReader(inData)));
		}

	///////////////////////////////////////////////////////////////////
	protected boolean getNextApplet(StreamTokenizer stream)
		throws IOException
		{
		/*
		Eat tokens from the tokenized stream until the next "<applet" string
		is found.
		*/

		//don't let the StreamTokenizer treat slashes or single quotes specially
		stream.ordinaryChar('/'); stream.ordinaryChar('\'');

		//eat tokens until end of file or "<applet" found
		int token = ' ', lastToken = ' ';
		while (! ((token == END_OF_FILE) ||
				((token == WORD) &&
				(lastToken == '<') &&
				stream.sval.toLowerCase().equals("applet"))))
			{
			lastToken = token;
			token = stream.nextToken();
			}

		//if i'm not at end of file, then i found an applet tag
		if (token != END_OF_FILE)
			return true;
		else
			return false;
		}

	///////////////////////////////////////////////////////////////////
	protected Vector parseAppletTokens(StreamTokenizer stream)
		throws IOException
		{
		/*
		Parse the stream into applet html tokens. Return a vector of all
		the strings found. The stream is currently positioned inside an
		applet html tag.
		*/

		//until end of file or "</applet" found:
		//while inside a param tag (or the first <applet..> tag)
		//store every number, quoted string, and word found
		Vector strings = new Vector();
		int token = ' ', lastToken = ' ', secondToLastToken = ' ';
		boolean insideTag = true;
		while (! ((token == END_OF_FILE) ||
				((token == WORD) &&
				(secondToLastToken == '<') && (lastToken == '/') &&
				stream.sval.toLowerCase().equals("applet"))))
			{
			secondToLastToken = lastToken;
			lastToken = token;
			token = stream.nextToken();

			//set a flag if i just got into a param tag
			if (!insideTag && (lastToken == '<') && (token == WORD) &&
				stream.sval.toLowerCase().equals("param"))
				insideTag = true;

			//clear the flag when i hit the end of the param tag
			if (insideTag && (token == '>'))
				insideTag = false;

			//if i'm in a param tag, save all numbers, quoted strings, and words
			//unless it happens to be "param" or a word preceded by "</"
			//(that word has to be checked at the top of the loop
			//to see if it's "applet")
			if (insideTag)
				{
				if (token == NUMBER)
					strings.addElement(String.valueOf(stream.nval));
				if (token == '"')
					strings.addElement(stream.sval);
				if ((token == WORD) && stream.sval.toLowerCase().equals("param"))
					continue;
				if ((token == WORD) && (lastToken != '/'))
					strings.addElement(stream.sval);
				}
			}

		return strings;
		}

	///////////////////////////////////////////////////////////////////
	protected Hashtable storeAppletParameters(Vector strings)
		{
		/*
		Put the string vector's entries into a symbol table of name-value pairs
		for each of the applet's parameters. Set any applet variables found
		and return the symbol table.

		NOTE: need to update to handle OBJECT ARCHIVE ALT NAME ALIGN VSPACE
		and HSPACE
		*/

		//if there's nothing to do, just return
		if (strings.size() == 0)
			return null;

		//the first two entries could be "codebase" and the applet's directory
		int i = 0;
		if (strings.elementAt(i).equals("codebase"))
			{
			appletCodebase = (String) strings.elementAt(i + 1);
			i += 2;
			}

		//the next two (or first two) should be "code" and the applet's name
		if (strings.elementAt(i).equals("code"))
			{
			appletName = (String) strings.elementAt(i + 1);
			i += 2;
			}

		//the next two should be "width" and the applet's width
		if (strings.elementAt(i).equals("width"))
			{
			appletWidth =
				Double.valueOf((String) strings.elementAt(i + 1)).intValue();
			i += 2;
			}

		//the next two should be "height" and the applet's height
		if (strings.elementAt(i).equals("height"))
			{
			appletHeight =
				Double.valueOf((String) strings.elementAt(i + 1)).intValue();
			i += 2;
			}

		//now fetch the (possibly empty) sequence of applet parameters:
		//the string "name" then a name, the string "value" then a value
		Hashtable appletParameters = new Hashtable();
		for (int j = i; j <= strings.size() - 4; j += 4)
			if (strings.elementAt(j).equals("name") &&
				strings.elementAt(j + 2).equals("value"))
				appletParameters.put(strings.elementAt(j + 1),
									strings.elementAt(j + 3));

		return appletParameters;
		}
	}
